# This file is part of Rubeyond.
#
# Rubeyond is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Rubeyond is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Rubeyond.  If not, see <http://www.gnu.org/licenses/>.

module Rubeyond

  # The +UUID+ class handles representing a universally unique identifier.
  #
  # ==== Examples
  #
  #   # Create a new unique UUID.
  #   @uuid = Rubeyond::UUID.create
  #
  #   # Create an instance of UUID from a uuid value.
  #   @uuid = Rubeyond::UUID.create :uuid => "fd0289a5-8eec-4a08-9283-81d02c9d2fff"
  #
  #   # Create an instance of UUID from an encoded value.
  #   @uuid = Rubeyond::UUID.create :encoded => 336307859334295828133695192821923655679
  class UUID

    # to enable comparison operators
    include Comparable

    # A string representation of the +UUID+.
    attr_reader :uuid

    # The encoded value of the +UUID+.
    attr_reader :encoded

    # Creates a new +UUID+ instance.
    #
    # If provided, it will create a new instances with the given
    # uuid value.
    #
    # ==== Attributes
    #
    # * +options+ - creation options
    #
    # ==== Options
    #
    # * +:uuid: - the +UUID+ value to use
    # * +:encoded: - an encoded +UUID+
    #
    def self.create(options = {})
      # if we have a uuid then create from that
      if options[:uuid]
        Rubeyond::UUID.new(options[:uuid])
      elsif options[:encoded]
        uuid = decode(options[:encoded])
        Rubeyond::UUID.new(uuid)
      else
        Rubeyond::UUID.new
      end
    end

    # Comparison operator.
    def ==(that) # :nodoc:
      self.uuid == that.uuid
    end

    # Spaceship operator.
    def <=>(that) # :nodoc:
      if self.uuid < that.uuid
        -1
      elsif self.uuid > that.uuid
        1
      else
        0
      end
    end

    # Returns a string representation of the UUID.
    def to_s
      @uuid
    end

    private

    # Private initializer to encourage a factory pattern.
    def initialize(uuid = nil) # :nodoc:
      if uuid.nil?
        @uuid = SecureRandom.uuid.upcase
      else
        @uuid = uuid.upcase
      end
      @encoded = UUID.encode(@uuid)
    end

    # Encodes the value of the UUID.
    def self.encode(uuid)
      # ensure that the UUID is in the right format
      # xxxxxxxx-xxxx-Mxxx-Nxxx-xxxxxxxxxxxx
      if uuid =~ /[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}/
        (uuid[0, 8] + uuid[9, 4] + uuid[14, 4] + uuid[19, 4] + uuid[24, 12]).to_i(16)
      else
        raise ArgumentError, "Invalid UUID: #{@uuid}"
      end
    end

    # Decodes the encoded UUID value.
    def self.decode(uuid) # :nodoc:
      result = ("%032x" % uuid).upcase
      result.insert(8, "-").insert(13, "-").insert(18, "-").insert(23, "-")
    end

  end

end
